﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Excel = Microsoft.Office.Interop.Excel;
using System.Reflection;
/*
 * Author: Simon Reed
 * Date: 18/02/09
 * Description: Reads .dat files produced by pedometer devices and allows simple processing
 *              of that data and output into a number of formats
 */


namespace CarnegieDataAnalysis
{
    public partial class DataAnalysis : Form
    {
        private DataPoint[] machineData;    //Stores the datapoints for use in the program
        private string serialNumber = "";   //Stores the serial number of the device
        private DateTime startTime;         //Stores the start date and time for the initial data
        private DateTime epochPeriod;       //Stores the epoch period in the hh:mm:ss part
        private DateTime downloadTime;      //Stores the download time from the header
        private String[] headerData = new String[10];//Stores the header info
        private DataPoint[] workingData;    //Temp stores the data that is being processed
        private AboutBox aboutbox = new AboutBox();
        //private GraphBox graphBox = new GraphBox();
        private String saveFileName;        //Stores the filename for the save function
        private String loadFileName;        //Stores the filename for the load function

        public DataAnalysis()
        {
            InitializeComponent();
        }

        /*  Sets the startTime and endTime pickers to the same value
         */
        private void Form1_Load(object sender, EventArgs e)
        {
            DateTime startTimes = new DateTime(DateTime.Now.Year,DateTime.Now.Month,DateTime.Now.Day,DateTime.Now.Hour,0,0);
            startTimePicker.Value = startTimes;
            endTimePicker.Value = startTimes;
        }

        /*  Checks that the file exists and runs the File Load background worker
         */
        private void openDialog_FileOk(object sender, CancelEventArgs e)
        {
            button1.Enabled = false;
            button2.Enabled = false;
            loadToolStripMenuItem.Enabled = false;
            exportToolStripMenuItem.Enabled = false;
            optionsPanel.Enabled = false;
            if (!File.Exists(openDialog.FileName))
            //check the file exists before going further
            {
                System.Windows.Forms.MessageBox.Show("File does not exist", "Error",
                    System.Windows.Forms.MessageBoxButtons.OK,
                    System.Windows.Forms.MessageBoxIcon.Error);
                button1.Enabled = true;
                button2.Enabled = true;
                exportToolStripMenuItem.Enabled = true;
                loadToolStripMenuItem.Enabled = true;
                return;
            }
            loadFileName = openDialog.FileName;
            loadProgressBar.Value = 0;
            loadProgressBar.Maximum = 100;
            backgroundWorkerFileLoad.WorkerReportsProgress = true;
            backgroundWorkerFileLoad.RunWorkerAsync();
        }
        
        /*  Sets the initial directory of the open file dialog to the current
         *  working directory and then shows the dialog
         */
        private void button1_Click(object sender, EventArgs e)
        {
            openDialog.InitialDirectory = System.Environment.CurrentDirectory;
            openDialog.ShowDialog();
        }

        /*  Trys to load a file (loadFileName) and create an array of header info and datapoints
         */
        private void backgroundWorkerFileLoad_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                //create a new file reader and stream reader
                FileStream fileReader = new FileStream
                    (loadFileName, FileMode.Open, FileAccess.Read);
                StreamReader readstream = new StreamReader(fileReader);
                String headLine1 = readstream.ReadLine().Trim();
                //check the first line of the header contains ActiGraph
                if (!headLine1.Contains("ActiGraph"))
                {
                    MessageBox.Show("The file is not in the correct format", "Error",
                    System.Windows.Forms.MessageBoxButtons.OK,
                    System.Windows.Forms.MessageBoxIcon.Error);
                    e.Result = "1";
                    return;
                }
                
                //read the serial number from the second header line
                String headLine2 = readstream.ReadLine().Trim();
                serialNumber = headLine2.Substring(15);
                //read the start time into a temp string for use later
                String headLine3 = readstream.ReadLine().Trim();
                String tempTime = headLine3.Substring(11);
                //read the start date into a temp string for use later
                String headLine4 = readstream.ReadLine().Trim();
                String tempDateYear = headLine4.Substring(headLine4.Length - 4);
                String tempDateDay = headLine4.Substring(headLine4.Length - 7, 2);
                String tempDateMonth = headLine4.Substring(headLine4.Length - 10, 2);
                String tempDate = tempDateDay + "/" + tempDateMonth + "/" + tempDateYear;
                //parse the date and time from the file into a DateTime for the start time
                startTime = DateTime.Parse(tempDate + " " + tempTime);
                //read the epoch time into a temp string for use later
                String headLine5 = readstream.ReadLine().Trim();
                String tempEpochPeriod = headLine5.Substring(24);
                //parse the epoch time with a date into a DateTime for the epoch time
                epochPeriod = DateTime.Parse("1/1/1 " + tempEpochPeriod);
                //read the final 5 lines
                //read the download time
                String headLine6 = readstream.ReadLine().Trim();
                String tempDownloadTime = headLine6.Substring(headLine6.Length - 8);
                //read the download date
                String headLine7 = readstream.ReadLine().Trim();
                tempDateYear = headLine7.Substring(headLine7.Length - 4);
                tempDateDay = headLine7.Substring(headLine7.Length - 7, 2);
                tempDateMonth = headLine7.Substring(headLine7.Length - 10, 2);
                String tempDownloadDate = tempDateDay + "/" + tempDateMonth + "/" + tempDateYear;
                downloadTime = DateTime.Parse(tempDownloadDate + " " + tempDownloadTime);
                String headLine8 = readstream.ReadLine().Trim();
                String headLine9 = readstream.ReadLine().Trim();
                String headLine10 = readstream.ReadLine().Trim();
                headerData[1] = headLine2;
                headerData[2] = headLine3;
                headerData[3] = headLine4; 
                headerData[4] = headLine5;
                headerData[5] = headLine6; 
                headerData[6] = headLine7;
                headerData[7] = headLine8; 
                headerData[8] = headLine9;
                headerData[9] = headLine10;
                String[] dataStringArray = new String[fileReader.Length / 8];
                int dataStringArrayPos = 0;
                while (!readstream.EndOfStream)
                {
                    //read the rest of the file into a string array, one line per item
                    dataStringArray[dataStringArrayPos] = readstream.ReadLine();
                    dataStringArrayPos++;
                    #region More Broken Code
                    /*(//read the rest of the file in 8 char blocks into tempBuffer
                    Char[] tempBuffer = new Char[8];
                    readstream.Read(tempBuffer, 0, 8);
                    //create a string from the buffered data
                    String tempString;
                    tempString = new string(tempBuffer);
                    //trim the whitespace from the string
                    tempString.Trim();
                    int dataPointValue;
                    //parse the string into int
                    dataPointValue = Convert.ToInt32(tempString);
                    //Int32.TryParse(tempString.Trim(), out dataPointValue);
                    //add item to DataIntArray
                    dataIntArray[dataArrayPos] = dataPointValue;
                    dataArrayPos++;
                    //report back the percentage of the file processed
                    //backgroundWorkerFileLoad.ReportProgress((int)(fileReader.Position / fileReader.Length * 100));*/
                    #endregion
                }
                //create a counter for the position in the integer array
                int dataArrayPos = 0;
                //intialise the dataIntArray to /8 of the file length
                int[] dataIntArray = new int[fileReader.Length / 8];
                for (int x = 0; x <= dataStringArrayPos; x++)
                {
                    if (!(dataStringArray[x] == null))
                    {
                        //find the number of 8s in the line length
                        int valsPerLine = dataStringArray[x].Length / 8;
                        for (int y = 0; y < valsPerLine; y++)
                        {
                            String tempSubString = dataStringArray[x].Substring((y * 8), 8);
                            tempSubString = tempSubString.Trim();
                            int dataPointValue = Convert.ToInt32(tempSubString);
                            dataIntArray[dataArrayPos] = dataPointValue;
                            dataArrayPos++;
                        }
                    }
                }
                    backgroundWorkerFileLoad.ReportProgress(50);
                //initalise machineData with the total number of data points
                machineData = new DataPoint[dataArrayPos];
                for (int x = 0; x < dataArrayPos; x++)
                {
                    //create the DateTime for the current data point
                    DateTime dataPointTime = new DateTime(startTime.Ticks + (x * epochPeriod.TimeOfDay.Ticks));
                    //create a new DataPoint using the DateTime and int from dataIntArray
                    DataPoint newDataPoint = new DataPoint(dataPointTime, dataIntArray[x]);
                    //store the DataPoint in machineData
                    machineData[x] = newDataPoint;
                    //report back the percentage of the file processed
                    //backgroundWorkerFileLoad.ReportProgress((int)(x / dataArrayPos * 100));
                }
                backgroundWorkerFileLoad.ReportProgress(100);
                e.Result = "0";
            }
            catch
            {
                MessageBox.Show("The file is not in the correct format", "Error",
                    System.Windows.Forms.MessageBoxButtons.OK,
                    System.Windows.Forms.MessageBoxIcon.Error);
                e.Result = "1";
            }
            
        }
        
        /*  Updates the progress bar as the File Load background worker
         *  reports progress changed
         */
        private void backgroundWorkerFileLoad_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.loadProgressBar.Value = e.ProgressPercentage;
        }
        
        /*  Runs when the File Load background worker completes
         *  If the file loads successfully the header box is updated
         */
        private void backgroundWorkerFileLoad_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (!(e.Result.ToString() == "1"))
            {
                MessageBox.Show("File load completed. \n" + machineData.Length + " datapoints loaded", "Operation Complete",
                        System.Windows.Forms.MessageBoxButtons.OK,
                        System.Windows.Forms.MessageBoxIcon.Information);
                headerInfoBox.Text = "Start Time: " + startTime.DayOfWeek + " " + startTime.ToString() + " \n";
                headerInfoBox.Text = headerInfoBox.Text + "Epoch Period: " + epochPeriod.ToLongTimeString() + " \n";
                headerInfoBox.Text = headerInfoBox.Text + "End Time: " +
                    machineData[machineData.Length - 1].getDate().DayOfWeek + " " +
                    machineData[machineData.Length - 1].getDate().ToString() + " \n";
                headerInfoBox.Text = headerInfoBox.Text + "Downloaded: " + downloadTime.ToString() + " \n";
                headerInfoBox.Text = headerInfoBox.Text + "Datapoints: " + machineData.Length + " \n";
                headerInfoBox.Text = headerInfoBox.Text + "Serial Number: " + serialNumber + " \n";
                headerInfoBox.Refresh();
            }
            button1.Enabled = true;
            button2.Enabled = true;
            exportToolStripMenuItem.Enabled = true;
            loadToolStripMenuItem.Enabled = true;
            optionsPanel.Enabled = true;
            loadProgressBar.Value = 0;
            identifyDayStartEnd(machineData);
        }

        /*  Shows the File Save dialogue
         */
        private void button2_Click(object sender, EventArgs e)
        {
            String fileNameStr = serialNumber + "-" + startTime.Year
                + startTime.Month + startTime.Day + ".csv";
            saveFileDialog1.FileName = fileNameStr;
            saveFileDialog1.ShowDialog();
        }

        /*  Disables appropriate buttons and runs the File Exporter background worker
         */
        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            button1.Enabled = false;
            button2.Enabled = false;
            loadToolStripMenuItem.Enabled = false;
            exportToolStripMenuItem.Enabled = false;
            optionsPanel.Enabled = false;
            saveFileName = saveFileDialog1.FileName;
            backgroundWorkerFileExportCSV.WorkerReportsProgress = true;
            backgroundWorkerFileExportCSV.RunWorkerAsync();
        }

        /*  Trys to gain access to the file system to output the contents of machineData
         *  as a CSV file
         *  Result = 1 if task completes
         */
        private void backgroundWorkerFileExportCSV_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                FileStream fileSave = new
                    FileStream(saveFileName, FileMode.Create, FileAccess.Write);
                StreamWriter stream = new StreamWriter(fileSave);
                stream.WriteLine("Day of the week, Year, Month, Day, Hour, Minute, Second, Activity value");
                int reportEvery = machineData.Length / 100;
                int percentageComplete = 0;
                for (int x = 0; x < machineData.Length; x++)
                {
                    String streamString = machineData[x].getDate().DayOfWeek + " , ";
                    streamString += machineData[x].getDate().Year + " , ";
                    streamString += machineData[x].getDate().Month + " , ";
                    streamString += machineData[x].getDate().Day + " , ";
                    streamString += machineData[x].getDate().Hour + " , ";
                    streamString += machineData[x].getDate().Minute + " , ";
                    streamString += machineData[x].getDate().Second + " , ";
                    streamString += machineData[x].getValue();
                    stream.WriteLine(streamString);
                    if (x == (percentageComplete + 1) * reportEvery)
                    {
                        percentageComplete++;
                        backgroundWorkerFileExportCSV.ReportProgress(percentageComplete);

                    }
                }
                stream.Close();
                stream.Dispose();
                fileSave.Dispose();
                e.Result = 1;
            }
            catch (Exception f)
            {
                e.Result = f.Message;
            }
        }

        /*  Updates the progress bar as the File Export CSV background worker
         *  reports progress changed
         */
        private void backgroundWorkerFileExportCSV_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.loadProgressBar.Value = e.ProgressPercentage;
        }

        /*  Runs when the File Export CSV completes
         *  Dependant of the success of the tasks 1 of 2 messages is shown to the user
         */
        private void backgroundWorkerFileExportCSV_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            DialogResult response = DialogResult.Ignore;
            if (e.Result.ToString() == "1")
            {
                response = MessageBox.Show("File save completed. \n Would you like to open that file now?", "Operation Complete",
                        System.Windows.Forms.MessageBoxButtons.YesNo,
                        System.Windows.Forms.MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("File save failed \n" + e.Result, "Operation Failed",
                        System.Windows.Forms.MessageBoxButtons.OK,
                        System.Windows.Forms.MessageBoxIcon.Error);
            }
            loadProgressBar.Value = 0;
            optionsPanel.Enabled = true;
            button1.Enabled = true;
            button2.Enabled = true;
            loadToolStripMenuItem.Enabled = true;
            exportToolStripMenuItem.Enabled = true;
            if (response == DialogResult.Yes)
            {
                System.Diagnostics.Process.Start(saveFileDialog1.FileName);
            }
        }

        /*  If the aboutBox is not shown then it is shown
         */
        private void aboutMenu_Click(object sender, EventArgs e)
        {
            if (aboutbox.Visible != true)
            {
                aboutbox = new AboutBox();
                aboutbox.Show();
            }
        }

        /*  Checks to see if the first datapoint has been set to 0 and then runs the
         *  Data Processor background worker
         */
        private void processButton_Click(object sender, EventArgs e)
        {
            if (machineData[0].getDate() == startTime)
            {
                machineData[0] = new DataPoint(startTime, 0);
            }
            processButton.Enabled = false;
            backgroundWorkerDataProcessor.WorkerReportsProgress = true;
            backgroundWorkerDataProcessor.RunWorkerAsync();
        }

        /*  Closes all forms and ends all processing
         */
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        /*  Input:  mins - The number of mins of 0 to remove
         *          startData - Array of DataPoints to be processed
         *  Output: Array of DataPoints with the chosen number of minutes of 0 removed
         */
        private DataPoint[] clearConsecutiveZero(int mins, DataPoint[] startData)
        {
            DataPoint[] workingData = new DataPoint[startData.Length];
            int maxConsecutiveZero = (int)(numericUpDown1.Value * (int)(60 / epochPeriod.TimeOfDay.TotalSeconds));
            int consecutiveZero = 0;
            bool clearZeros = true;
            int newDataLength = 0;
            for (int currentPointer = 0; currentPointer < startData.Length; currentPointer++)
            {
                if (consecutiveZero > maxConsecutiveZero && !clearZeros)
                {
                    newDataLength = newDataLength - consecutiveZero;
                    clearZeros = true;
                }
                if (startData[currentPointer].getValue() == 0)
                {
                    consecutiveZero++;
                }
                else
                {
                    clearZeros = false;
                    consecutiveZero = 0;
                }
                if (!clearZeros)
                {
                    workingData[newDataLength] = startData[currentPointer];
                    newDataLength++;
                    clearZeros = false;
                }
            }
            
            DataPoint[] finishedData = new DataPoint[newDataLength];
            for (int x = 0; x < newDataLength; x++)
            {
                finishedData[x] = workingData[x];
            }
            return finishedData;
        }

        /*  Checks which processing methods have been enabled and runs the appropriate
         *  method
         */
        private void backgroundWorkerDataProcessor_DoWork(object sender, DoWorkEventArgs e)
        {
            workingData = new DataPoint[machineData.Length];
            machineData.CopyTo(workingData, 0);
            workingData = clearGreaterThan(workingData, (short)numericUpDown3.Value);
            if (!(startTimePicker.Value == endTimePicker.Value))
            {
                workingData = clearBetweenTimes(workingData, startTimePicker.Value, endTimePicker.Value);
            }
            if (!(numericUpDown1.Value == 0))
            {
                workingData = clearConsecutiveZero((int)numericUpDown1.Value, workingData);
            }
            workingData = clearSelectedDays(workingData);
            int dataPointsRemoved = machineData.Length - workingData.Length;
        }

        /*  Runs when the process background worker completes and reports the number of
         *  DataPoints that have been removed during the last processing run
         */
        private void backgroundWorkerDataProcessor_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            int dataPointsRemoved = machineData.Length - workingData.Length;
            MessageBox.Show("Processing complete. \n" + dataPointsRemoved +
                " data points have been removed", "Processing Complete",
                        System.Windows.Forms.MessageBoxButtons.OK);
            machineData = workingData;
            loadProgressBar.Value = 0;
            identifyDayStartEnd(machineData);
            processButton.Enabled = true;
        }

        /*  Updates the progress bar with the percentage complete reported
         */
        private void backgroundWorkerDataProcessor_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.loadProgressBar.Value = e.ProgressPercentage;
        }

        /*  Input:  Array of DataPoints
         *  Output: Int of the number of days contained in the array
         */
        private int getNumberOfDays(DataPoint[] inputData)
        {
            TimeSpan totalLength = new TimeSpan(inputData[inputData.Length - 1].getDate().Ticks - inputData[0].getDate().Ticks);
            int numberofdays = totalLength.Days + 1;
            return numberofdays;
        }

        /*  Input:  Array of DataPoints to be processed
         *  Output: Array of DataPoints with only those DataPoints whos day was selected included
         */
        private DataPoint[] clearSelectedDays(DataPoint[] startData)
        {
            DataPoint[] workingData = new DataPoint[startData.Length];
            bool[] daysToInclude = new bool[7];
            //When DayOfWeek is cast to int 0 = Sunday, 1 = Monday .... 6 = Saturday
            daysToInclude[0] = daySun.Checked;
            daysToInclude[1] = dayMon.Checked;
            daysToInclude[2] = dayTues.Checked;
            daysToInclude[3] = dayWed.Checked;
            daysToInclude[4] = dayThur.Checked;
            daysToInclude[5] = dayFri.Checked;
            daysToInclude[6] = daySat.Checked;
            int daysIncluded = 0;
            for (int x = 0; x < startData.Length; x++)
            {
                if (daysToInclude[(int)startData[x].getDate().DayOfWeek])
                {
                    workingData[daysIncluded] = startData[x];
                    daysIncluded++;
                }
            }
            DataPoint[] finishedData = new DataPoint[daysIncluded];
            for (int x = 0; x < daysIncluded; x++)
            {
                finishedData[x] = workingData[x];
            }

            return finishedData;
        }

        /*  Input:  startData - Array of DataPoints to be processed
         *          maxValue - The maximum value any datapoint should contain
         *  Output: Array of DataPoints where any value that was greater than maxValue
         *          has been set to 0
         */
        private DataPoint[] clearGreaterThan(DataPoint[] startData, short maxValue)
        {
            for (int x = 0; x < startData.Length; x++)
            {
                startData[x].cleanGreaterThan(maxValue);
            }

            return startData;
        }

        /*  Input:  startData - Array of DataPoints to be processed
         *          startTime - The start of the period to be excluded(date part is ignored)
         *          endTime - The end of the period to be excluded(date part is ignored)
         *  Output: Array of DataPoints where DataPoints that fell between the 2 times have
         *          a value of 0
         */
        private DataPoint[] clearBetweenTimes(DataPoint[] startData, DateTime startTime, DateTime endTime)
        {
            for (int x = 0; x < startData.Length; x++)
            {
                startData[x].cleanBetweenTimes(startTime,endTime);
            }

            return startData;

        }

        /*  Input:  Array of DataPoints to be processed
         * 
         *  Fills the checkedListBox1 with the start and end times of each day and also
         *  uses the vaild day value to determine the validity of each day, checking the 
         *  box if the day is vaild
         */
        private void identifyDayStartEnd(DataPoint[] data)
        {
            TimeSpan[] dayLengths = new TimeSpan[getNumberOfDays(data) + 1];
            DateTime[] dayStarts = new DateTime[getNumberOfDays(data) + 1];
            int[] arrayDayItems = new int[getNumberOfDays(data) + 1];
            DayOfWeek lastDay = data[0].getDate().DayOfWeek;
            dayStarts[0] = data[0].getDate();
            int nextDayStart = 1;
            int lastDayStartArray = 0;
            for (int x = 1; x < data.Length; x++)
            {
                if (!(data[x].getDate().DayOfWeek == lastDay))
                {
                    dayStarts[nextDayStart] = data[x].getDate();
                    dayLengths[nextDayStart - 1] = data[x - 1].getDate() - dayStarts[nextDayStart - 1];
                    arrayDayItems[nextDayStart - 1] = x - 1 - lastDayStartArray;
                    lastDayStartArray = x;
                    lastDay = data[x].getDate().DayOfWeek;
                    nextDayStart++;
                }
            }
            
            dayLengths[nextDayStart - 1] = data[data.Length - 1].getDate() - dayStarts[nextDayStart - 1];
            arrayDayItems[nextDayStart - 1] = data.Length - 1 - lastDayStartArray;
            checkedListBox1.Items.Clear();
            TimeSpan validDayLength = new TimeSpan((int)numericUpDown2.Value,0,0);
            for (int x = 0; x < nextDayStart; x++)
            {
                string itemString;
                itemString = dayStarts[x].Date.ToString("dddd") + " " + dayStarts[x].ToShortDateString();
                itemString += " Starts - ";
                itemString += dayStarts[x].TimeOfDay.ToString();
                itemString += " Ends - ";
                DateTime endOfDay = dayStarts[x] + dayLengths[x];
                itemString += endOfDay.TimeOfDay.ToString();
                TimeSpan dataLength = new TimeSpan(dayLengths[x].Ticks);
                bool isValid = false;
                if (dataLength >= validDayLength)
                {
                    isValid = true;
                }
                checkedListBox1.Items.Add(itemString, isValid);
            }
        }

        /*  Input:  Array of DataPoints to be output
         *  Attempts to load Microsoft Excel and output the data as requested by the client
         *  Checks the version of Execl that has been opened, if less than version 12 only
         *  65595 rows are output on each sheet
         */
        private void exportExcel(DataPoint[] startData)
        {
            if (startData.Length < 1)
            {
                return;
            }
            //Based on http://support.microsoft.com/kb/302084
            TimeSpan[] dayLengths = new TimeSpan[getNumberOfDays(startData) + 1];
            DateTime[] dayStarts = new DateTime[getNumberOfDays(startData) + 1];
            int[] arrayDayItems = new int[getNumberOfDays(startData) + 1];
            DayOfWeek lastDay = startData[0].getDate().DayOfWeek;
            dayStarts[0] = startData[0].getDate();
            bool[] validDayArray = new bool[getNumberOfDays(startData) + 1];
            int[] sedentryCount = new int[getNumberOfDays(startData) + 1];
            int[] lightCount = new int[getNumberOfDays(startData) + 1];
            int[] moderateCount = new int[getNumberOfDays(startData) + 1];
            int[] vigCount = new int[getNumberOfDays(startData) + 1];
            int[] dayTotalCount = new int[getNumberOfDays(startData) + 1];
            bool[] isWeekend = new bool[getNumberOfDays(startData) + 1];
            int sedentryVal = (int)numericUpDown4.Value;
            int lightVal = (int)numericUpDown5.Value;
            int moderateVal = (int)numericUpDown6.Value;
            int nextDayStart = 1;
            int lastDayStartArray = 0;
            for (int x = 1; x < startData.Length; x++)
            {
                if (!(startData[x].getDate().DayOfWeek == lastDay))
                {
                    dayStarts[nextDayStart] = startData[x].getDate();
                    dayLengths[nextDayStart - 1] = startData[x - 1].getDate() - dayStarts[nextDayStart - 1];
                    arrayDayItems[nextDayStart - 1] = x - lastDayStartArray;
                    lastDayStartArray = x;
                    lastDay = startData[x].getDate().DayOfWeek;
                    nextDayStart++;
                }
                int dataVal = startData[x].getValue();
                if (dataVal <= sedentryVal)
                {
                    sedentryCount[nextDayStart - 1]++;
                }
                else if (dataVal > sedentryVal && dataVal <= lightVal)
                {
                    lightCount[nextDayStart - 1]++;
                }
                else if (dataVal > lightVal && dataVal <= moderateVal)
                {
                    moderateCount[nextDayStart - 1]++;
                }
                else
                {
                    vigCount[nextDayStart - 1]++;
                }
                dayTotalCount[nextDayStart - 1] = dayTotalCount[nextDayStart - 1] + dataVal;
            }
            dayLengths[nextDayStart - 1] = startData[startData.Length - 1].getDate() - dayStarts[nextDayStart - 1];
            arrayDayItems[nextDayStart - 1] = startData.Length - lastDayStartArray;
            checkedListBox1.Items.Clear();
            TimeSpan validDayLength = new TimeSpan((int)numericUpDown2.Value, 0, 0);
            for (int x = 0; x < nextDayStart; x++)
            {
                string itemString;
                itemString = dayStarts[x].Date.ToString("dddd") + " " + dayStarts[x].ToShortDateString();
                itemString += " Starts - ";
                itemString += dayStarts[x].TimeOfDay.ToString();
                itemString += " Ends - ";
                DateTime endOfDay = dayStarts[x] + dayLengths[x];
                itemString += endOfDay.TimeOfDay.ToString();
                TimeSpan dataLength = new TimeSpan(epochPeriod.TimeOfDay.Ticks * arrayDayItems[x]);
                bool isValid = false;
                validDayArray[x] = false;
                if (dataLength >= validDayLength)
                {
                    isValid = true;
                    validDayArray[x] = true;
                }
                if (((int)dayStarts[x].Date.DayOfWeek == 0) || ((int)dayStarts[x].Date.DayOfWeek == 6))
                {
                    isWeekend[x] = true;
                }
                else
                {
                    isWeekend[x] = false;
                }

                checkedListBox1.Items.Add(itemString, isValid);
            }
            Excel.Application oXL;
            Excel._Workbook oWB;
            Excel._Worksheet[] oSheet = new Excel._Worksheet[dayLengths.Length*2];
            try
            {
                //Start Excel and get Application object.
                oXL = new Excel.Application();
                oXL.Visible = true;
                oXL.UserControl = false;

                int sheetsused = 0;
                #region Sheet1 Containing All Data
                //Get a new workbook.
                oWB = (Excel._Workbook)(oXL.Workbooks.Add(1));
                sheetsused++;
                oSheet[0] = (Excel._Worksheet)oWB.ActiveSheet;
                oSheet[0].Name = "All Data";
                //Add table headers going cell by cell.
                oSheet[0].Cells[1, 1] = "D.O.W";
                oSheet[0].Cells[1, 2] = "Day";
                oSheet[0].Cells[1, 3] = "Month";
                oSheet[0].Cells[1, 4] = "Year";
                oSheet[0].Cells[1, 5] = "Hour";
                oSheet[0].Cells[1, 6] = "Minute";
                oSheet[0].Cells[1, 7] = "Second";
                oSheet[0].Cells[1, 8] = "Value";

                //Format A1:H1 as bold, vertical alignment = center.
                oSheet[0].get_Range("A1", "H1").Font.Bold = true;
                oSheet[0].get_Range("A1", "H1").VerticalAlignment =
                    Excel.XlVAlign.xlVAlignCenter;

                string[,] dayNames = new string[startData.Length, 1];
                int[,] otherVals = new int[startData.Length, 7];
                int maximumCells;
                int excelVersion = 0;
                Int32.TryParse(oXL.Version, out excelVersion);
                if (excelVersion <= 11 && startData.Length > 65536)
                {
                    maximumCells = 65534;
                }
                else
                {
                    maximumCells = startData.Length;
                }

                for (int x = 0; x < maximumCells; x++)
                {
                    dayNames[x, 0] = startData[x].getDate().DayOfWeek + "";
                    otherVals[x, 0] = startData[x].getDate().Day;
                    otherVals[x, 1] = startData[x].getDate().Month;
                    otherVals[x, 2] = startData[x].getDate().Year;
                    otherVals[x, 3] = startData[x].getDate().Hour;
                    otherVals[x, 4] = startData[x].getDate().Minute;
                    otherVals[x, 5] = startData[x].getDate().Second;
                    otherVals[x, 6] = startData[x].getValue();
                }
                //Fill Col A with Day Names

                oSheet[0].get_Range("A2", "A" + (maximumCells + 1)).Value2 = dayNames;

                //Fill the rest

                oSheet[0].get_Range("B2", "H" + (maximumCells + 1)).Value2 = otherVals;

                #endregion
                #region Data on one sheet per day
                int curPos = 0;
                for(int y = 1; y <= nextDayStart; y++)
                {
                    oSheet[y] = (Excel._Worksheet)oWB.Worksheets.Add(Missing.Value, oSheet[y - 1], Missing.Value, Missing.Value);
                    sheetsused++;
                    oSheet[y].Name = dayStarts[y - 1].DayOfWeek.ToString() + " " + dayStarts[y - 1].ToLongDateString() + " DATA";
                    //Add table headers going cell by cell.
                    oSheet[y].Cells[1, 1] = "Time";
                    oSheet[y].Cells[1, 2] = "Value";

                    //Format A1:D1 as bold, vertical alignment = center.
                    oSheet[y].get_Range("A1", "B1").Font.Bold = true;
                    oSheet[y].get_Range("A1", "B1").VerticalAlignment =
                        Excel.XlVAlign.xlVAlignCenter;
                    dayNames = new string[startData.Length, 1];
                    otherVals = new int[startData.Length, 1];
                    excelVersion = 0;
                    Int32.TryParse(oXL.Version, out excelVersion);
                    if (excelVersion <= 11 && arrayDayItems[y-1] > 65536)
                    {
                        maximumCells = 65534;
                    }
                    else
                    {
                        maximumCells = arrayDayItems[y-1];
                    }
                    oSheet[y].get_Range("A2", "A" + (maximumCells + 1)).NumberFormat = "hh:mm:ss";
                    for (int x = 0; x < maximumCells; x++)
                    {
                        dayNames[x, 0] = startData[curPos].getDate().TimeOfDay.ToString();
                        otherVals[x, 0] = startData[curPos].getValue();
                        curPos++;
                    }
                    //Fill Col A with Day Names

                    oSheet[y].get_Range("A2", "A" + (maximumCells + 1)).Value2 = dayNames;

                    //Fill the rest

                    oSheet[y].get_Range("B2", "B" + (maximumCells + 1)).Value2 = otherVals;
                }
                #endregion
                #region Data Analysis Sheet
                oSheet[sheetsused] = (Excel._Worksheet)oWB.Worksheets.Add(Missing.Value, oSheet[sheetsused - 1], Missing.Value, Missing.Value);
                oSheet[sheetsused].Name = "Data Analysis";
                String[,] basicInfo = new String [2,6];
                basicInfo[0, 0] = "Instrument Serial Number";
                basicInfo[0, 1] = "Start Time";
                basicInfo[0, 2] = "Start Date";
                basicInfo[0, 3] = "Epoch Period";
                basicInfo[0, 4] = "Download Time";
                basicInfo[0, 5] = "Download Date";
                basicInfo[1, 0] = serialNumber;
                basicInfo[1, 1] = startData[0].getDate().ToLongTimeString();
                basicInfo[1, 2] = startData[0].getDate().ToLongDateString();
                basicInfo[1, 3] = epochPeriod.ToLongTimeString();
                basicInfo[1, 4] = downloadTime.ToLongTimeString();
                basicInfo[1, 5] = downloadTime.ToLongDateString();
                oSheet[sheetsused].get_Range("A1", "F2").Value2 = basicInfo;
                oSheet[sheetsused].get_Range("A1", "F1").Font.Bold = true;
                oSheet[sheetsused].get_Range("A1", "F1").VerticalAlignment =
                    Excel.XlVAlign.xlVAlignCenter;
                oSheet[sheetsused].get_Range("A1", "F2").EntireColumn.AutoFit();
                sheetsused++;
                int currentSheetLine = 0;
                #region Day by day analysis
                String[,] dayAnalysis = new String[(nextDayStart * 4), 12];
                
                
                for (int x = 0; x < nextDayStart; x++)
                {
                    //Blank line before each day
                    for (int y = 0; y < 12; y++)
                    {
                        dayAnalysis[currentSheetLine, y] = "";
                    }
                    currentSheetLine++;
                    //Add the Day on the next line
                    dayAnalysis[currentSheetLine,0] = dayStarts[x].DayOfWeek.ToString();
                    for (int y = 1; y < 12; y++)
                    {
                        dayAnalysis[currentSheetLine,y ] = "";
                    }
                    currentSheetLine++;
                    //Add the titles for the table
                    dayAnalysis[currentSheetLine, 0] = "Valid (Minimum Wear Time)";
                    dayAnalysis[currentSheetLine, 1] = "Start Time";
                    dayAnalysis[currentSheetLine, 2] = "Finish Time";
                    dayAnalysis[currentSheetLine, 3] = "Number of Valid Counts";
                    dayAnalysis[currentSheetLine, 4] = "Valid Total Counts";
                    dayAnalysis[currentSheetLine, 5] = "Valid Minutes Worn";
                    dayAnalysis[currentSheetLine, 6] = "Valid Average Counts";
                    dayAnalysis[currentSheetLine, 7] = "n Sedentary Counts";
                    dayAnalysis[currentSheetLine, 8] = "n Light Counts";
                    dayAnalysis[currentSheetLine, 9] = "n Moderate Counts";
                    dayAnalysis[currentSheetLine, 10] = "n Vigorous Counts";
                    dayAnalysis[currentSheetLine, 11] = "n Moderate/Vigorous Counts";
                    currentSheetLine++;
                    //Add the data for the table
                    if (validDayArray[x])
                    {
                        dayAnalysis[currentSheetLine, 0] = "YES";
                    }
                    else
                    {
                        dayAnalysis[currentSheetLine, 0] = "NO";
                    }
                    dayAnalysis[currentSheetLine, 1] = dayStarts[x].TimeOfDay.ToString();
                    DateTime endOfDay = dayStarts[x] + dayLengths[x];
                    dayAnalysis[currentSheetLine, 2] = endOfDay.TimeOfDay.ToString();
                    dayAnalysis[currentSheetLine, 3] = arrayDayItems[x].ToString();
                    dayAnalysis[currentSheetLine, 4] = dayTotalCount[x].ToString();
                    TimeSpan minsWorn = new TimeSpan(0, 0, (int)(arrayDayItems[x] * epochPeriod.TimeOfDay.TotalSeconds));
                    dayAnalysis[currentSheetLine, 5] = minsWorn.ToString();
                    dayAnalysis[currentSheetLine, 6] = (dayTotalCount[x] / arrayDayItems[x]).ToString();
                    dayAnalysis[currentSheetLine, 7] = sedentryCount[x].ToString();
                    dayAnalysis[currentSheetLine, 8] = lightCount[x].ToString();
                    dayAnalysis[currentSheetLine, 9] = moderateCount[x].ToString();
                    dayAnalysis[currentSheetLine, 10] = vigCount[x].ToString();
                    dayAnalysis[currentSheetLine, 11] = (moderateCount[x] + vigCount[x]).ToString();
                    currentSheetLine++;
                }

                oSheet[sheetsused-1].get_Range("A3", "L" + (2 +(4*nextDayStart))).Value2 = dayAnalysis;

                #endregion


                #endregion
                //Make sure Excel is visible and give the user control
                //of Microsoft Excel's lifetime.
                oXL.Visible = true;
                oXL.UserControl = true;
            }
            catch (Exception theException)
            {
                String errorMessage;
                errorMessage = "Error: ";
                errorMessage = String.Concat(errorMessage, theException.Message);
                errorMessage = String.Concat(errorMessage, " Line: ");
                errorMessage = String.Concat(errorMessage, theException.Source);

                MessageBox.Show(errorMessage, "Error");
            }
        }

        /*  Calls the exportExcel method using machineData as input
         */
        private void buttonGraph_Click(object sender, EventArgs e)
        {
            exportExcel(machineData);
        }

        /*  Refreshes the information in checkedListBox1
         */
        private void checkedListBox1_Validated(object sender, EventArgs e)
        {
            identifyDayStartEnd(machineData);
        }

        /*  Refreshes the information in checkedListBox1 when the valid day length is changed
         */
        private void numericUpDown2_ValueChanged(object sender, EventArgs e)
        {
            identifyDayStartEnd(machineData);
        }

        /* Displays the graphBox if it is not already visable
         */
        /*private void button3_Click(object sender, EventArgs e)
        {
            if (graphBox.Visible != true)
            {
                graphBox = new GraphBox();
                graphBox.Show();
            }
            graphBox.generateNewChart(generateGraphInputData(machineData));
        }*/

        /*  Input:  Array of DataPoints to be processed
         *  Output: 2D array day x DataPoints for that day
         */
        private DataPoint[][] generateGraphInputData(DataPoint[] inputData)
        {
            DataPoint[][] dividedData = new DataPoint[getNumberOfDays(inputData)+1][];
            //split input data into separate days
            DayOfWeek lastDay = inputData[0].getDate().DayOfWeek;
            int[] arrayDayItems = new int[getNumberOfDays(inputData) + 1];
            int dayNumber = 0;
            int dayPointer = 0;
            int lastDayStartArray = 0;
            dividedData[0] = new DataPoint[inputData.Length];
            for (int x = 0; x < inputData.Length; x++)
            {
                if (!(inputData[x].getDate().DayOfWeek == lastDay))
                {
                    dayNumber++;
                    dividedData[dayNumber] = new DataPoint[inputData.Length];
                    arrayDayItems[dayNumber - 1] = x - lastDayStartArray;
                    lastDay = inputData[x].getDate().DayOfWeek;
                    lastDayStartArray = x;
                    dayPointer = 0;
                }
                dividedData[dayNumber][dayPointer] = inputData[x];
                dayPointer++;
            }
            arrayDayItems[dayNumber] = inputData.Length - lastDayStartArray;

            DataPoint[][] finishedData = new DataPoint[dayNumber+1][];
            for (int x = 0; x <= dayNumber; x++)
            {
                finishedData[x] = new DataPoint[arrayDayItems[x]];
                for (int y = 0; y < arrayDayItems[x]; y++)
                {
                    finishedData[x][y] = dividedData[x][y];
                }
            }
            
            return finishedData;
        }

        /*  Begin the drop process for the header box
         */
        private void headerInfoBox_DragEnter(object sender, DragEventArgs e)
        {
            // make sure they're actually dropping files (not text or anything else)
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true)
                // allow them to continue
                // (without this, the cursor stays a "NO" symbol
                e.Effect = DragDropEffects.All;
        }

        /*  Complete the drop process for the header box
         *  Check the file exists then call the FileLoad background worker
         */
        private void headerInfoBox_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            if (!File.Exists(files[0]))
            //check the file exists before going further
            {
                System.Windows.Forms.MessageBox.Show("File does not exist", "Error",
                    System.Windows.Forms.MessageBoxButtons.OK,
                    System.Windows.Forms.MessageBoxIcon.Error);
                button1.Enabled = true;
                button2.Enabled = true;
                exportToolStripMenuItem.Enabled = true;
                loadToolStripMenuItem.Enabled = true;
                return;
            }
            button1.Enabled = false;
            button2.Enabled = false;
            loadToolStripMenuItem.Enabled = false;
            exportToolStripMenuItem.Enabled = false;
            optionsPanel.Enabled = false;
            loadFileName = files[0];
            loadProgressBar.Value = 0;
            loadProgressBar.Maximum = 100;
            backgroundWorkerFileLoad.WorkerReportsProgress = true;
            backgroundWorkerFileLoad.RunWorkerAsync();
        }

        /*  Update the labels based on the new value
         */
        private void numericUpDown4_ValueChanged(object sender, EventArgs e)
        {
            label7.Text = "Light Range > " + (numericUpDown4.Value) + " <=";
            numericUpDown5.Minimum = numericUpDown4.Value + 1;
        }

        /*  Update the labels based on the new value
         */
        private void numericUpDown5_ValueChanged(object sender, EventArgs e)
        {
            label8.Text = "Moderate Range > " + (numericUpDown5.Value) + " <=";
            numericUpDown6.Minimum = numericUpDown5.Value + 1;
        }

        /*  Update the labels based on the new value
         */
        private void numericUpDown6_ValueChanged(object sender, EventArgs e)
        {
            label9.Text = "Vigorous Range > " + (numericUpDown6.Value);
        }

        private void button3_Click(object sender, EventArgs e)
        {

        }
    }
}
