﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Excel = Microsoft.Office.Interop.Excel;
using System.Reflection;
/*
 * Author: Simon Reed
 * Date: 28/01/09
 */


namespace CarnegieDataAnalysis
{
    public partial class Form1 : Form
    {
        private DataPoint[] machineData;
        private string serialNumber = "";
        private DateTime startTime;
        private DateTime epochPeriod;
        private DateTime downloadTime;
        private String[] headerData = new String[10];
        private DataPoint[] workingData;
        AboutBox aboutbox = new AboutBox();
        GraphBox graphBox = new GraphBox();

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            graphBox.Show();
            DateTime startTimes = new DateTime(DateTime.Now.Year,DateTime.Now.Month,DateTime.Now.Day,DateTime.Now.Hour,0,0);
            startTimePicker.Value = startTimes;
            endTimePicker.Value = startTimes;
        }

        private void openDialog_FileOk(object sender, CancelEventArgs e)
        {
            button1.Enabled = false;
            button2.Enabled = false;
            loadToolStripMenuItem.Enabled = false;
            exportToolStripMenuItem.Enabled = false;
            optionsPanel.Enabled = false;
            if (!File.Exists(openDialog.FileName))
            //check the file exists before going further
            {
                System.Windows.Forms.MessageBox.Show("File does not exist", "Error",
                    System.Windows.Forms.MessageBoxButtons.OK,
                    System.Windows.Forms.MessageBoxIcon.Error);
                button1.Enabled = true;
                button2.Enabled = true;
                exportToolStripMenuItem.Enabled = true;
                loadToolStripMenuItem.Enabled = true;
                return;
            }
            loadProgressBar.Value = 0;
            loadProgressBar.Maximum = 100;
            backgroundWorkerFileLoad.WorkerReportsProgress = true;
            backgroundWorkerFileLoad.RunWorkerAsync();
        }
        
        private void cleanData()
        {
            DateTime startTime = new DateTime(1,1,1,3,0,0);
            DateTime endTime = new DateTime(1,1,1,7,0,0);
            int maxConsecutiveZero = 40 * 30;
            int consecutiveZero = 0;
            bool clearZero = false;
            for(int x = 0; x < machineData.Length; x++)
            {
                machineData[x].cleanBetweenTimes(startTime, endTime);
                machineData[x].cleanGreaterThan(300);
                if (machineData[x].getValue() == 0)
                {
                    if(!clearZero)
                    consecutiveZero++;
                }
                else
                {
                    clearZero = false;
                    consecutiveZero = 0;
                }
                if (consecutiveZero >= maxConsecutiveZero)
                {
                    clearZero = true;
                    x = x - maxConsecutiveZero;
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            openDialog.InitialDirectory = System.Environment.CurrentDirectory;
            openDialog.ShowDialog();
        }

        private void backgroundWorkerFileLoad_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                //create a new file reader and stream reader
                FileStream fileReader = new FileStream
                    (openDialog.FileName, FileMode.Open, FileAccess.Read);
                StreamReader readstream = new StreamReader(fileReader);
                String headLine1 = readstream.ReadLine().Trim();
                //check the first line of the header contains ActiGraph
                if (!headLine1.Contains("ActiGraph"))
                {
                    MessageBox.Show("The file is not in the correct format", "Error",
                    System.Windows.Forms.MessageBoxButtons.OK,
                    System.Windows.Forms.MessageBoxIcon.Error);
                    return;
                }
                
                //read the serial number from the second header line
                String headLine2 = readstream.ReadLine().Trim();
                serialNumber = headLine2.Substring(15);
                //read the start time into a temp string for use later
                String headLine3 = readstream.ReadLine().Trim();
                String tempTime = headLine3.Substring(11);
                //read the start date into a temp string for use later
                String headLine4 = readstream.ReadLine().Trim();
                String tempDateYear = headLine4.Substring(headLine4.Length - 4);
                String tempDateDay = headLine4.Substring(headLine4.Length - 7, 2);
                String tempDateMonth = headLine4.Substring(headLine4.Length - 10, 2);
                String tempDate = tempDateDay + "/" + tempDateMonth + "/" + tempDateYear;
                //parse the date and time from the file into a DateTime for the start time
                startTime = DateTime.Parse(tempDate + " " + tempTime);
                //read the epoch time into a temp string for use later
                String headLine5 = readstream.ReadLine().Trim();
                String tempEpochPeriod = headLine5.Substring(24);
                //parse the epoch time with a date into a DateTime for the epoch time
                epochPeriod = DateTime.Parse("1/1/1 " + tempEpochPeriod);
                //read the final 5 lines
                //read the download time
                String headLine6 = readstream.ReadLine().Trim();
                String tempDownloadTime = headLine6.Substring(headLine6.Length - 8);
                //read the download date
                String headLine7 = readstream.ReadLine().Trim();
                tempDateYear = headLine7.Substring(headLine7.Length - 4);
                tempDateDay = headLine7.Substring(headLine7.Length - 7, 2);
                tempDateMonth = headLine7.Substring(headLine7.Length - 10, 2);
                String tempDownloadDate = tempDateDay + "/" + tempDateMonth + "/" + tempDateYear;
                downloadTime = DateTime.Parse(tempDownloadDate + " " + tempDownloadTime);
                String headLine8 = readstream.ReadLine().Trim();
                String headLine9 = readstream.ReadLine().Trim();
                String headLine10 = readstream.ReadLine().Trim();
                headerData[1] = headLine2;
                headerData[2] = headLine3;
                headerData[3] = headLine4; 
                headerData[4] = headLine5;
                headerData[5] = headLine6; 
                headerData[6] = headLine7;
                headerData[7] = headLine8; 
                headerData[8] = headLine9;
                headerData[9] = headLine10;
                String[] dataStringArray = new String[fileReader.Length / 8];
                int dataStringArrayPos = 0;
                while (!readstream.EndOfStream)
                {
                    //read the rest of the file into a string array, one line per item
                    dataStringArray[dataStringArrayPos] = readstream.ReadLine();
                    dataStringArrayPos++;
                    #region More Broken Code
                    /*(//read the rest of the file in 8 char blocks into tempBuffer
                    Char[] tempBuffer = new Char[8];
                    readstream.Read(tempBuffer, 0, 8);
                    //create a string from the buffered data
                    String tempString;
                    tempString = new string(tempBuffer);
                    //trim the whitespace from the string
                    tempString.Trim();
                    int dataPointValue;
                    //parse the string into int
                    dataPointValue = Convert.ToInt32(tempString);
                    //Int32.TryParse(tempString.Trim(), out dataPointValue);
                    //add item to DataIntArray
                    dataIntArray[dataArrayPos] = dataPointValue;
                    dataArrayPos++;
                    //report back the percentage of the file processed
                    //backgroundWorkerFileLoad.ReportProgress((int)(fileReader.Position / fileReader.Length * 100));*/
                    #endregion
                }
                //create a counter for the position in the integer array
                int dataArrayPos = 0;
                //intialise the dataIntArray to /8 of the file length
                int[] dataIntArray = new int[fileReader.Length / 8];
                for (int x = 0; x <= dataStringArrayPos; x++)
                {
                    if (!(dataStringArray[x] == null))
                    {
                        //find the number of 8s in the line length
                        int valsPerLine = dataStringArray[x].Length / 8;
                        for (int y = 0; y < valsPerLine; y++)
                        {
                            String tempSubString = dataStringArray[x].Substring((y * 8), 8);
                            tempSubString = tempSubString.Trim();
                            int dataPointValue = Convert.ToInt32(tempSubString);
                            dataIntArray[dataArrayPos] = dataPointValue;
                            dataArrayPos++;
                        }
                    }
                }
                    backgroundWorkerFileLoad.ReportProgress(50);
                //initalise machineData with the total number of data points
                machineData = new DataPoint[dataArrayPos];
                for (int x = 0; x < dataArrayPos; x++)
                {
                    //create the DateTime for the current data point
                    DateTime dataPointTime = new DateTime(startTime.Ticks + (x * epochPeriod.TimeOfDay.Ticks));
                    //create a new DataPoint using the DateTime and int from dataIntArray
                    DataPoint newDataPoint = new DataPoint(dataPointTime, dataIntArray[x]);
                    //store the DataPoint in machineData
                    machineData[x] = newDataPoint;
                    //report back the percentage of the file processed
                    //backgroundWorkerFileLoad.ReportProgress((int)(x / dataArrayPos * 100));
                }
                backgroundWorkerFileLoad.ReportProgress(100);
            }
            catch
            {
                MessageBox.Show("The file is not in the correct format", "Error",
                    System.Windows.Forms.MessageBoxButtons.OK,
                    System.Windows.Forms.MessageBoxIcon.Error);
            }
            
        }

        private void backgroundWorkerFileLoad_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.loadProgressBar.Value = e.ProgressPercentage;
        }

        private void backgroundWorkerFileLoad_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            MessageBox.Show("File load completed. \n" + machineData.Length + " datapoints loaded", "Operation Complete",
                    System.Windows.Forms.MessageBoxButtons.OK,
                    System.Windows.Forms.MessageBoxIcon.Information);
            headerInfoBox.Text = "Start Time: " + startTime.DayOfWeek + " " + startTime.ToString() + " \n";
            headerInfoBox.Text = headerInfoBox.Text + "Epoch Period: " + epochPeriod.ToLongTimeString() + " \n";
            headerInfoBox.Text = headerInfoBox.Text + "End Time: " +
                machineData[machineData.Length - 1].getDate().DayOfWeek + " " +
                machineData[machineData.Length - 1].getDate().ToString() + " \n";
            headerInfoBox.Text = headerInfoBox.Text + "Downloaded: " + downloadTime.ToString() + " \n";
            headerInfoBox.Text = headerInfoBox.Text + "Datapoints: " + machineData.Length + " \n";
            headerInfoBox.Text = headerInfoBox.Text + "Serial Number: " + serialNumber + " \n";
            headerInfoBox.Refresh();
            button1.Enabled = true;
            button2.Enabled = true;
            exportToolStripMenuItem.Enabled = true;
            loadToolStripMenuItem.Enabled = true;
            optionsPanel.Enabled = true;
            loadProgressBar.Value = 0;
            identifyDayStartEnd(machineData);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            String fileNameStr = serialNumber + "-" + startTime.Year
                + startTime.Month + startTime.Day + ".csv";
            saveFileDialog1.FileName = fileNameStr;
            saveFileDialog1.ShowDialog();
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            button1.Enabled = false;
            button2.Enabled = false;
            loadToolStripMenuItem.Enabled = false;
            exportToolStripMenuItem.Enabled = false;
            optionsPanel.Enabled = false;
            backgroundWorkerFileExportCSV.WorkerReportsProgress = true;
            backgroundWorkerFileExportCSV.RunWorkerAsync();
        }

        private void backgroundWorkerFileExportCSV_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                FileStream fileSave = new
                    FileStream(saveFileDialog1.FileName, FileMode.Create, FileAccess.Write);
                StreamWriter stream = new StreamWriter(fileSave);
                stream.WriteLine("Day of the week, Year, Month, Day, Hour, Minute, Second, Activity value");
                int reportEvery = machineData.Length / 100;
                int percentageComplete = 0;
                for (int x = 0; x < machineData.Length; x++)
                {
                    String streamString = machineData[x].getDate().DayOfWeek + " , ";
                    streamString += machineData[x].getDate().Year + " , ";
                    streamString += machineData[x].getDate().Month + " , ";
                    streamString += machineData[x].getDate().Day + " , ";
                    streamString += machineData[x].getDate().Hour + " , ";
                    streamString += machineData[x].getDate().Minute + " , ";
                    streamString += machineData[x].getDate().Second + " , ";
                    streamString += machineData[x].getValue();
                    stream.WriteLine(streamString);
                    if (x == (percentageComplete + 1) * reportEvery)
                    {
                        percentageComplete++;
                        backgroundWorkerFileExportCSV.ReportProgress(percentageComplete);

                    }
                }
                stream.Close();
                stream.Dispose();
                fileSave.Dispose();
                e.Result = 1;
            }
            catch (Exception f)
            {
                e.Result = f.Message;
            }
        }

        private void backgroundWorkerFileExportCSV_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.loadProgressBar.Value = e.ProgressPercentage;
        }

        private void backgroundWorkerFileExportCSV_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            DialogResult response = DialogResult.Ignore;
            if (e.Result.ToString() == "1")
            {
                response = MessageBox.Show("File save completed. \n Would you like to open that file now?", "Operation Complete",
                        System.Windows.Forms.MessageBoxButtons.YesNo,
                        System.Windows.Forms.MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("File save failed \n" + e.Result, "Operation Failed",
                        System.Windows.Forms.MessageBoxButtons.OK,
                        System.Windows.Forms.MessageBoxIcon.Error);
            }
            loadProgressBar.Value = 0;
            optionsPanel.Enabled = true;
            button1.Enabled = true;
            button2.Enabled = true;
            loadToolStripMenuItem.Enabled = true;
            exportToolStripMenuItem.Enabled = true;
            if (response == DialogResult.Yes)
            {
                System.Diagnostics.Process.Start(saveFileDialog1.FileName);
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
           // MessageBox.Show("Thank you for using this software " + System.Environment.UserName, "Goodbye",
           //             System.Windows.Forms.MessageBoxButtons.OK);
            
        }

        private void aboutMenu_Click(object sender, EventArgs e)
        {
            if (aboutbox.Visible != true)
            {
                aboutbox = new AboutBox();
                aboutbox.Show();
            }
        }

        private void processButton_Click(object sender, EventArgs e)
        {
            if (machineData[0].getDate() == startTime)
            {
                machineData[0].setValue(0);
            }
            processButton.Enabled = false;
            backgroundWorkerDataProcessor.WorkerReportsProgress = true;
            backgroundWorkerDataProcessor.RunWorkerAsync();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private DataPoint[] clearConsecutiveZero(int mins, DataPoint[] startData)
        {
            DataPoint[] workingData = new DataPoint[startData.Length];
            int maxConsecutiveZero = (int)(numericUpDown1.Value * (int)(60 / epochPeriod.TimeOfDay.TotalSeconds));
            int consecutiveZero = 0;
            bool clearZeros = false;
            int newDataLength = 0;
            for (int currentPointer = 0; currentPointer < startData.Length; currentPointer++)
            {
                if (consecutiveZero > maxConsecutiveZero && !clearZeros)
                {
                    newDataLength = newDataLength - consecutiveZero;
                    clearZeros = true;
                }
                if (startData[currentPointer].getValue() == 0)
                {
                    consecutiveZero++;
                }
                else
                {
                    clearZeros = false;
                    consecutiveZero = 0;
                }
                if (!clearZeros)
                {
                    workingData[newDataLength] = startData[currentPointer];
                    newDataLength++;
                    clearZeros = false;
                }
            }
            
            DataPoint[] finishedData = new DataPoint[newDataLength];
            for (int x = 0; x < newDataLength; x++)
            {
                finishedData[x] = workingData[x];
            }
            return finishedData;
        }

        private void backgroundWorkerDataProcessor_DoWork(object sender, DoWorkEventArgs e)
        {
            workingData = new DataPoint[machineData.Length];
            machineData.CopyTo(workingData, 0);
            workingData = clearGreaterThan(workingData, (short)numericUpDown3.Value);
            if (!(startTimePicker.Value == endTimePicker.Value))
            {
                workingData = clearBetweenTimes(workingData, startTimePicker.Value, endTimePicker.Value);
            }
            if (!(numericUpDown1.Value == 0))
            {
                workingData = clearConsecutiveZero((int)numericUpDown1.Value, workingData);
            }
            workingData = clearSelectedDays(workingData);
            int dataPointsRemoved = machineData.Length - workingData.Length;
        }

        private void backgroundWorkerDataProcessor_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            int dataPointsRemoved = machineData.Length - workingData.Length;
            MessageBox.Show("Processing complete. \n" + dataPointsRemoved +
                " data points have been removed", "Processing Complete",
                        System.Windows.Forms.MessageBoxButtons.OK);
            machineData = workingData;
            loadProgressBar.Value = 0;
            identifyDayStartEnd(machineData);
            processButton.Enabled = true;
        }

        private void backgroundWorkerDataProcessor_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.loadProgressBar.Value = e.ProgressPercentage;
        }

        private int getNumberOfDays(DataPoint[] inputData)
        {
            TimeSpan totalLength = new TimeSpan(inputData[inputData.Length - 1].getDate().Ticks - startTime.Ticks);
            int numberofdays = totalLength.Days + 1;
            return numberofdays;
        }

        private DataPoint[] clearSelectedDays(DataPoint[] startData)
        {
            DataPoint[] workingData = new DataPoint[startData.Length];
            bool[] daysToInclude = new bool[7];
            //When DayOfWeek is cast to int 0 = Sunday, 1 = Monday .... 6 = Saturday
            daysToInclude[0] = daySun.Checked;
            daysToInclude[1] = dayMon.Checked;
            daysToInclude[2] = dayTues.Checked;
            daysToInclude[3] = dayWed.Checked;
            daysToInclude[4] = dayThur.Checked;
            daysToInclude[5] = dayFri.Checked;
            daysToInclude[6] = daySat.Checked;
            int daysIncluded = 0;
            for (int x = 0; x < startData.Length; x++)
            {
                if (daysToInclude[(int)startData[x].getDate().DayOfWeek])
                {
                    workingData[daysIncluded] = startData[x];
                    daysIncluded++;
                }
            }
            DataPoint[] finishedData = new DataPoint[daysIncluded];
            for (int x = 0; x < daysIncluded; x++)
            {
                finishedData[x] = workingData[x];
            }

            return finishedData;
        }

        private DataPoint[] clearGreaterThan(DataPoint[] startData, short maxValue)
        {
            for (int x = 0; x < startData.Length; x++)
            {
                startData[x].cleanGreaterThan(maxValue);
            }

            return startData;
        }

        private DataPoint[] clearBetweenTimes(DataPoint[] startData, DateTime startTime, DateTime endTime)
        {
            for (int x = 0; x < startData.Length; x++)
            {
                startData[x].cleanBetweenTimes(startTime,endTime);
            }

            return startData;

        }

        private void identifyDayStartEnd(DataPoint[] data)
        {
            TimeSpan[] dayLengths = new TimeSpan[getNumberOfDays(data) + 1];
            DateTime[] dayStarts = new DateTime[getNumberOfDays(data) + 1];
            int[] arrayDayItems = new int[getNumberOfDays(data) + 1];
            DayOfWeek lastDay = data[0].getDate().DayOfWeek;
            dayStarts[0] = data[0].getDate();
            int nextDayStart = 1;
            int lastDayStartArray = 0;
            for (int x = 1; x < data.Length; x++)
            {
                if (!(data[x].getDate().DayOfWeek == lastDay))
                {
                    dayStarts[nextDayStart] = data[x].getDate();
                    dayLengths[nextDayStart - 1] = data[x - 1].getDate() - dayStarts[nextDayStart - 1];
                    arrayDayItems[nextDayStart - 1] = x - 1 - lastDayStartArray;
                    lastDayStartArray = x;
                    lastDay = data[x].getDate().DayOfWeek;
                    nextDayStart++;
                }
            }
            
            dayLengths[nextDayStart - 1] = data[data.Length - 1].getDate() - dayStarts[nextDayStart - 1];
            arrayDayItems[nextDayStart - 1] = data.Length - 1 - lastDayStartArray;
            checkedListBox1.Items.Clear();
            TimeSpan validDayLength = new TimeSpan((int)numericUpDown2.Value,0,0);
            for (int x = 0; x < nextDayStart; x++)
            {
                string itemString;
                itemString = dayStarts[x].Date.ToString("dddd") + " " + dayStarts[x].ToShortDateString();
                itemString += " Starts - ";
                itemString += dayStarts[x].TimeOfDay.ToString();
                itemString += " Ends - ";
                DateTime endOfDay = dayStarts[x] + dayLengths[x];
                itemString += endOfDay.TimeOfDay.ToString();
                TimeSpan dataLength = new TimeSpan(dayLengths[x].Ticks);
                bool isValid = false;
                if (dataLength >= validDayLength)
                {
                    isValid = true;
                }
                checkedListBox1.Items.Add(itemString, isValid);
            }
        }

        /*private void button3_Click(object sender, EventArgs e)
        {
            button3.Text = "" + getNumberOfDays(machineData);
            identifyDayStartEnd(machineData);
            /*if (graphbox.Visible != true)
            {
                graphbox = new GraphBox();
                graphbox.Show();
            }
            /*Excel.Application oXL;
            Excel._Workbook oWB;
            Excel._Worksheet oSheet;
            Excel.Range oRng;

            try
            {
                //Start Excel and get Application object.
                oXL = new Excel.Application();
                oXL.Visible = true;
                oXL.UserControl = false;

                //Get a new workbook.
                oWB = (Excel._Workbook)(oXL.Workbooks.Add(Missing.Value));
                oSheet = (Excel._Worksheet)oWB.ActiveSheet;

                //Add table headers going cell by cell.
                oSheet.Cells[1, 1] = "First Name";
                oSheet.Cells[1, 2] = "Last Name";
                oSheet.Cells[1, 3] = "Full Name";
                oSheet.Cells[1, 4] = "Salary";

                //Format A1:D1 as bold, vertical alignment = center.
                oSheet.get_Range("A1", "D1").Font.Bold = true;
                oSheet.get_Range("A1", "D1").VerticalAlignment =
                    Excel.XlVAlign.xlVAlignCenter;

                // Create an array to multiple values at once.
                string[,] saNames = new string[5, 2];

                saNames[0, 0] = "John";
                saNames[0, 1] = "Smith";
                saNames[1, 0] = "Tom";
                saNames[1, 1] = "Brown";
                saNames[2, 0] = "Sue";
                saNames[2, 1] = "Thomas";
                saNames[3, 0] = "Jane";
                saNames[3, 1] = "Jones";
                saNames[4, 0] = "Adam";
                saNames[4, 1] = "Johnson";

                //Fill A2:B6 with an array of values (First and Last Names).
                oSheet.get_Range("A2", "B6").Value2 = saNames;

                //Make sure Excel is visible and give the user control
                //of Microsoft Excel's lifetime.
                oXL.Visible = true;
                oXL.UserControl = true;
            }
            catch (Exception theException)
            {
                String errorMessage;
                errorMessage = "Error: ";
                errorMessage = String.Concat(errorMessage, theException.Message);
                errorMessage = String.Concat(errorMessage, " Line: ");
                errorMessage = String.Concat(errorMessage, theException.Source);

                MessageBox.Show(errorMessage, "Error");
            }
        }*/

        private void exportExcel(DataPoint[] startData)
        {
            //Based on http://support.microsoft.com/kb/302084
            TimeSpan[] dayLengths = new TimeSpan[getNumberOfDays(startData) + 1];
            DateTime[] dayStarts = new DateTime[getNumberOfDays(startData) + 1];
            int[] arrayDayItems = new int[getNumberOfDays(startData) + 1];
            DayOfWeek lastDay = startData[0].getDate().DayOfWeek;
            dayStarts[0] = startData[0].getDate();
            int nextDayStart = 1;
            int lastDayStartArray = 0;
            for (int x = 1; x < startData.Length; x++)
            {
                if (!(startData[x].getDate().DayOfWeek == lastDay))
                {
                    dayStarts[nextDayStart] = startData[x].getDate();
                    dayLengths[nextDayStart - 1] = startData[x - 1].getDate() - dayStarts[nextDayStart - 1];
                    arrayDayItems[nextDayStart - 1] = x - lastDayStartArray;
                    lastDayStartArray = x;
                    lastDay = startData[x].getDate().DayOfWeek;
                    nextDayStart++;
                }
            }

            dayLengths[nextDayStart - 1] = startData[startData.Length - 1].getDate() - dayStarts[nextDayStart - 1];
            arrayDayItems[nextDayStart - 1] = startData.Length - lastDayStartArray;
            checkedListBox1.Items.Clear();
            TimeSpan validDayLength = new TimeSpan((int)numericUpDown2.Value, 0, 0);
            for (int x = 0; x < nextDayStart; x++)
            {
                string itemString;
                itemString = dayStarts[x].Date.ToString("dddd") + " " + dayStarts[x].ToShortDateString();
                itemString += " Starts - ";
                itemString += dayStarts[x].TimeOfDay.ToString();
                itemString += " Ends - ";
                DateTime endOfDay = dayStarts[x] + dayLengths[x];
                itemString += endOfDay.TimeOfDay.ToString();
                TimeSpan dataLength = new TimeSpan(epochPeriod.TimeOfDay.Ticks * arrayDayItems[x]);
                bool isValid = false;
                if (dataLength >= validDayLength)
                {
                    isValid = true;
                }
                checkedListBox1.Items.Add(itemString, isValid);
            }
            Excel.Application oXL;
            Excel._Workbook oWB;
            Excel._Worksheet[] oSheet = new Excel._Worksheet[dayLengths.Length*2];
            Excel.Range oRng;
            try
            {
                //Start Excel and get Application object.
                oXL = new Excel.Application();
                oXL.Visible = true;
                oXL.UserControl = false;
                #region Sheet1 Containing All Data
                //Get a new workbook.
                oWB = (Excel._Workbook)(oXL.Workbooks.Add(1));
                oSheet[0] = (Excel._Worksheet)oWB.ActiveSheet;
                oSheet[0].Name = "All Data";
                //Add table headers going cell by cell.
                oSheet[0].Cells[1, 1] = "D.O.W";
                oSheet[0].Cells[1, 2] = "Day";
                oSheet[0].Cells[1, 3] = "Month";
                oSheet[0].Cells[1, 4] = "Year";
                oSheet[0].Cells[1, 5] = "Hour";
                oSheet[0].Cells[1, 6] = "Minute";
                oSheet[0].Cells[1, 7] = "Second";
                oSheet[0].Cells[1, 8] = "Value";

                //Format A1:H1 as bold, vertical alignment = center.
                oSheet[0].get_Range("A1", "H1").Font.Bold = true;
                oSheet[0].get_Range("A1", "H1").VerticalAlignment =
                    Excel.XlVAlign.xlVAlignCenter;

                string[,] dayNames = new string[startData.Length, 1];
                int[,] otherVals = new int[startData.Length, 7];
                int maximumCells;
                int excelVersion = 0;
                Int32.TryParse(oXL.Version, out excelVersion);
                if (excelVersion <= 11 && startData.Length > 65536)
                {
                    maximumCells = 65534;
                }
                else
                {
                    maximumCells = startData.Length;
                }

                for (int x = 0; x < maximumCells; x++)
                {
                    dayNames[x, 0] = startData[x].getDate().DayOfWeek + "";
                    otherVals[x, 0] = startData[x].getDate().Day;
                    otherVals[x, 1] = startData[x].getDate().Month;
                    otherVals[x, 2] = startData[x].getDate().Year;
                    otherVals[x, 3] = startData[x].getDate().Hour;
                    otherVals[x, 4] = startData[x].getDate().Minute;
                    otherVals[x, 5] = startData[x].getDate().Second;
                    otherVals[x, 6] = startData[x].getValue();
                }
                //Fill Col A with Day Names

                oSheet[0].get_Range("A2", "A" + (maximumCells + 1)).Value2 = dayNames;

                //Fill the rest

                oSheet[0].get_Range("B2", "H" + (maximumCells + 1)).Value2 = otherVals;

                #endregion
                #region Data on one sheet per day
                int curPos = 0;
                for(int y = 1; y <= nextDayStart; y++)
                {
                    oSheet[y] = (Excel._Worksheet)oWB.Worksheets.Add(Missing.Value, oSheet[y - 1], Missing.Value, Missing.Value);
                    oSheet[y].Name = dayStarts[y - 1].DayOfWeek.ToString() + " " + dayStarts[y - 1].ToLongDateString() + " DATA";
                    //Add table headers going cell by cell.
                    oSheet[y].Cells[1, 1] = "Time";
                    oSheet[y].Cells[1, 2] = "Value";

                    //Format A1:D1 as bold, vertical alignment = center.
                    oSheet[y].get_Range("A1", "B1").Font.Bold = true;
                    oSheet[y].get_Range("A1", "B1").VerticalAlignment =
                        Excel.XlVAlign.xlVAlignCenter;
                    dayNames = new string[startData.Length, 1];
                    otherVals = new int[startData.Length, 1];
                    excelVersion = 0;
                    Int32.TryParse(oXL.Version, out excelVersion);
                    if (excelVersion <= 11 && arrayDayItems[y-1] > 65536)
                    {
                        maximumCells = 65534;
                    }
                    else
                    {
                        maximumCells = arrayDayItems[y-1];
                    }
                    oSheet[y].get_Range("A2", "A" + (maximumCells + 1)).NumberFormat = "hh:mm:ss";
                    for (int x = 0; x < maximumCells; x++)
                    {
                        dayNames[x, 0] = startData[curPos].getDate().TimeOfDay.ToString();
                        otherVals[x, 0] = startData[curPos].getValue();
                        curPos++;
                    }
                    //Fill Col A with Day Names

                    oSheet[y].get_Range("A2", "A" + (maximumCells + 1)).Value2 = dayNames;

                    //Fill the rest

                    oSheet[y].get_Range("B2", "B" + (maximumCells + 1)).Value2 = otherVals;
                }
                #endregion

                

                //Make sure Excel is visible and give the user control
                //of Microsoft Excel's lifetime.
                oXL.Visible = true;
                oXL.UserControl = true;
            }
            catch (Exception theException)
            {
                String errorMessage;
                errorMessage = "Error: ";
                errorMessage = String.Concat(errorMessage, theException.Message);
                errorMessage = String.Concat(errorMessage, " Line: ");
                errorMessage = String.Concat(errorMessage, theException.Source);

                MessageBox.Show(errorMessage, "Error");
            }
        }

        private void buttonGraph_Click(object sender, EventArgs e)
        {
            exportExcel(machineData);
        }

        private void checkedListBox1_Validated(object sender, EventArgs e)
        {
            identifyDayStartEnd(machineData);
        }

        private void numericUpDown2_ValueChanged(object sender, EventArgs e)
        {
            identifyDayStartEnd(machineData);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            graphBox.generateNewChart(generateGraphInputData(machineData));
        }

        private DataPoint[][] generateGraphInputData(DataPoint[] inputData)
        {
            DataPoint[][] dividedData = new DataPoint[getNumberOfDays(inputData)+1][];
            //split input data into separate days
            DayOfWeek lastDay = inputData[0].getDate().DayOfWeek;
            int[] arrayDayItems = new int[getNumberOfDays(inputData) + 1];
            int dayNumber = 0;
            int dayPointer = 0;
            int lastDayStartArray = 0;
            dividedData[0] = new DataPoint[inputData.Length];
            for (int x = 0; x < inputData.Length; x++)
            {
                if (!(inputData[x].getDate().DayOfWeek == lastDay))
                {
                    dayNumber++;
                    dividedData[dayNumber] = new DataPoint[inputData.Length];
                    arrayDayItems[dayNumber - 1] = x - lastDayStartArray;
                    lastDay = inputData[x].getDate().DayOfWeek;
                    lastDayStartArray = x;
                    dayPointer = 0;
                }
                dividedData[dayNumber][dayPointer] = inputData[x];
                dayPointer++;
            }
            arrayDayItems[dayNumber] = inputData.Length - lastDayStartArray;

            DataPoint[][] finishedData = new DataPoint[dayNumber+1][];
            for (int x = 0; x <= dayNumber; x++)
            {
                finishedData[x] = new DataPoint[arrayDayItems[x]];
                for (int y = 0; y < arrayDayItems[x]; y++)
                {
                    finishedData[x][y] = dividedData[x][y];
                }
            }
            
            return finishedData;
        }
    }
}
